package nl.iobyte.themepark.database;

import java.sql.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

public abstract class DB {

    public abstract Connection getConnection() throws SQLException;

    public abstract void closeConnection();

    public boolean execute(String query, HashMap<Integer, Object> objects) {
        if(query == null || query.isEmpty())
            return false;

        boolean b = false;
        try(Connection conn = getConnection()) {
            PreparedStatement statement = conn.prepareStatement(query);
            statement.closeOnCompletion();
            if(objects != null)
                for(Map.Entry<Integer, Object> entry : objects.entrySet())
                    statement.setObject(entry.getKey(), entry.getValue());

            b = statement.execute();
        } catch(Exception e) {
            e.printStackTrace();
        }
        
        return b;
    }

    public int executeUpdate(String query, HashMap<Integer, Object> objects) {
        if (query == null || query.isEmpty())
            return 0;

        int i = 0;
        try(Connection conn = getConnection()) {
            PreparedStatement statement = conn.prepareStatement(query);
            statement.closeOnCompletion();
            for (Map.Entry<Integer, Object> entry : objects.entrySet())
                statement.setObject(entry.getKey(), entry.getValue());

            i = statement.executeUpdate();
        } catch (Exception e) {
            e.printStackTrace();
        }
        
        return i;
    }

    public ArrayList<HashMap<String, Object>> executeQuery(String query, HashMap<Integer, Object> objects) {
        if (query == null || query.isEmpty())
            return null;

        ArrayList<HashMap<String, Object>> set = null;
        try(Connection conn = getConnection()) {
            PreparedStatement statement = conn.prepareStatement(query);
            statement.closeOnCompletion();
            if (objects != null)
                for (Map.Entry<Integer, Object> entry : objects.entrySet())
                    statement.setObject(entry.getKey(), entry.getValue());

            set = new ArrayList<>();

            ResultSet result = statement.executeQuery();
            ResultSetMetaData metadata = result.getMetaData();
            int columnCount = metadata.getColumnCount();
            while(result.next()) {
                HashMap<String, Object> map = new HashMap<>();
                for(int i = 1; i <= columnCount; i++)
                    map.put(metadata.getColumnName(i), result.getObject(i));

                set.add(map);
            }

            result.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        
        return set;
    }

}
