package nl.iobyte.themepark.database;

import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.HikariDataSource;
import nl.iobyte.themepark.ThemeParkPlugin;
import java.sql.Connection;
import java.sql.SQLException;

public class MySQL extends DB {

    private String dbName;
    private HikariDataSource source;

    /**
     * Initialize a new connection
     *
     * @param dbName The database name
     */
    public MySQL(String url, String host, int port, String dbName, String username, String password) {
        this.dbName = dbName;
        HikariConfig config = new HikariConfig();
        config.setPoolName(ThemeParkPlugin.getInstance().getName());
        config.setUsername(username);
        config.setPassword(password);
        config.setDriverClassName("com.mysql.jdbc.Driver");
        config.setMaximumPoolSize(10);
        config.setJdbcUrl(url.replace("%host%", host).replace("%port%", String.valueOf(port)).replace("%database%", dbName));
        source = new HikariDataSource(config);
    }

    /**
     * Get connection
     * @return Connection Instance
     */
    public Connection getConnection() throws SQLException {
        return source.getConnection();
    }

    /**
     * Close connection
     */
    public void closeConnection() {
        System.out.println("Closing the database connection for " + dbName + ".db!");
        source.close();
    }

}
