package nl.iobyte.themepark.ridecount;

import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import nl.iobyte.menuapi.map.WeakConcurrentHashMap;
import nl.iobyte.themepark.ThemeParkPlugin;
import nl.iobyte.themepark.api.API;
import nl.iobyte.themepark.api.attraction.Attraction;
import nl.iobyte.themepark.api.ridecount.AttractionCount;
import nl.iobyte.themepark.api.ridecount.CountManager;
import nl.iobyte.themepark.database.DB;
import nl.iobyte.themepark.database.DBManager;
import nl.iobyte.themepark.database.SQLite;
import nl.iobyte.workchain.components.Work;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.UUID;
import java.util.concurrent.CompletableFuture;

public class RideCountAPI {

    private static final WeakConcurrentHashMap<String, String> data = new WeakConcurrentHashMap<>(5 * 60 * 1000);
    private static final SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");

    public static CompletableFuture<Integer> getCount(String name, Attraction attraction) {
        CompletableFuture<Integer> fc = new CompletableFuture<>();
        getData(name).thenAccept(uuid -> {
            if(uuid == null) {
                fc.complete(-1);
                return;
            }

            getCount(uuid, attraction).thenAccept(fc::complete);
        });
        return fc;
    }

    public static CompletableFuture<Integer> getCount(UUID uuid, Attraction attraction) {
        CompletableFuture<Integer> fc = new CompletableFuture<>();
        if(uuid == null || attraction == null) {
            fc.complete(-1);
            return fc;
        }

        if(!API.isAttraction(attraction.getId())) {
            fc.complete(-1);
            return fc;
        }

        AttractionCount count = CountManager.getCounter(attraction.getId());
        if(count != null && count.hasCount(uuid)) {
            fc.complete(count.getCount(uuid).getCount());
            return fc;
        }

        Work.firstTask(() -> {
            DB db = DBManager.getDatabase("data");
            if(db == null)
                return null;

            HashMap<Integer, Object> map = new HashMap<>();
            map.put(1, uuid.toString());
            map.put(2, attraction.getId());
            map.put(3, simpleDateFormat.format(new Date()));

            return db.executeQuery("SELECT points FROM counts WHERE uuid=? AND attraction_id=? AND created_at=?", map);
        }).abortIfNull(() -> fc.complete(0)).nextTask(result -> {
            int i = 0;
            for(HashMap<String, Object> map : result) {
                int points = (int) map.get("points");
                if(points < 0) {
                    i = 0;
                    continue;
                }

                i = points;
            }

            return i;
        }).lastTask(fc::complete).setFullExecute(true).execute();
        return fc;
    }

    public static CompletableFuture<Boolean> addCount(String name, Attraction attraction, int amount) {
        CompletableFuture<Boolean> fc = new CompletableFuture<>();
        if(name == null || name.isEmpty() || attraction == null || amount < 0) {
            fc.complete(false);
            return fc;
        }

        if(!API.isAttraction(attraction.getId())) {
            fc.complete(false);
            return fc;
        }

        getData(name).thenAccept(uuid -> {
            if(uuid == null) {
                fc.complete(false);
                return;
            }

            addCount(uuid, attraction, amount).thenAccept(fc::complete);
        });
        return fc;
    }

    public static CompletableFuture<Boolean> addCount(UUID uuid, Attraction attraction, int amount) {
        CompletableFuture<Boolean> fc = new CompletableFuture<>();
        if(uuid == null || attraction == null) {
            fc.complete(false);
            return fc;
        }

        if(!API.isAttraction(attraction.getId())) {
            fc.complete(false);
            return fc;
        }

        AttractionCount count = CountManager.getCounter(attraction.getId());
        if(count != null)
            Bukkit.getScheduler().runTask(ThemeParkPlugin.getInstance(), () -> count.addCount(uuid, amount));

        Work.firstTask(() -> {
            SQLite db = (SQLite) DBManager.getDatabase("data");
            if(db == null)
                return null;


            String date = simpleDateFormat.format(new Date());

            boolean b = versionCompare(db.getVersion(), "3.24.0");
            if(!b) {
                HashMap<Integer, Object> map = new HashMap<>();
                map.put(1, uuid.toString());
                map.put(2, attraction.getId());
                map.put(3, amount);
                map.put(4, date);
                try {
                    b = db.execute("INSERT INTO counts(uuid, attraction_id, points,created_at) VALUES (?,?,?,?)", map);
                } catch (Exception e) {}

                if(b)
                    return true;

                map.put(3, map.get(2));
                map.put(2, map.get(1));
                map.put(1, amount);
                return db.executeUpdate("UPDATE counts SET points=? WHERE uuid=? AND attraction_id=? AND created_at=?", map) > 0;
            }

            HashMap<Integer, Object> map = new HashMap<>();
            map.put(1, uuid.toString());
            map.put(2, attraction.getId());
            map.put(3, amount);
            map.put(4, date);
            map.put(5, amount);

            return db.executeUpdate("INSERT INTO counts (uuid, attraction_id, points, created_at) VALUES (?, ?, ?, ?) ON CONFLICT(uuid, attraction_id, created_at) DO UPDATE SET points=points+?", map) > 0;
        }).abortIfNull(() -> fc.complete(false)).lastTask(fc::complete).setFullExecute(true).execute();
        return fc;
    }

    private static CompletableFuture<UUID> getData(String name) {
        CompletableFuture<UUID> fc = new CompletableFuture<>();
        Bukkit.getScheduler().runTaskAsynchronously(ThemeParkPlugin.getInstance(), () -> {
            if(data.containsKey(name)) {
                String str = data.get(name);
                if(str.isEmpty()) {
                    fc.complete(null);
                    return;
                }

                fc.complete(UUID.fromString(str));
                return;
            }

            Player player = Bukkit.getPlayer(name);
            if(player != null) {
                UUID uuid = player.getUniqueId();
                data.put(name, uuid.toString());
                fc.complete(uuid);
                return;
            }

            try {
                JsonObject object = sendGETRequestJSON("https://api.mojang.com/users/profiles/minecraft/" + name);
                if(object == null) {
                    data.put(name, "");
                    fc.complete(null);
                    return;
                }

                String id = object.get("id").getAsString();
                if(id == null || id.isEmpty()) {
                    data.put(name, "");
                    fc.complete(null);
                    return;
                }

                id = fixUUID(id);
                UUID uuid = UUID.fromString(id);
                data.put(name, id);
                fc.complete(uuid);
            } catch (Exception e) {
                data.put(name, "");
                fc.complete(null);
            }
        });

        return fc;
    }

    public static CompletableFuture<HashMap<UUID, Integer>> getTop(Attraction attraction, boolean week) {
        CompletableFuture<HashMap<UUID, Integer>> fc = new CompletableFuture<>();
        if(attraction == null) {
            fc.complete(null);
            return fc;
        }

        if(!API.isAttraction(attraction.getId())) {
            System.out.println("[ThemePark] Unknown attraction with ID: "+attraction.getId());
            fc.complete(null);
            return fc;
        }

        Work.firstTask(() -> {
            DB db = DBManager.getDatabase("data");
            if(db == null)
                return null;

            String timeSort = week ? "strftime('%W', created_at)=strftime('%W', 'now')" : "strftime('%j', created_at)=strftime('%j', 'now')";
            HashMap<Integer, Object> map = new HashMap<>();
            map.put(1, attraction.getId());

            return db.executeQuery("SELECT uuid, SUM(points) AS count FROM counts WHERE attraction_id=? AND "+timeSort+" AND strftime('%Y', created_at)==strftime('%Y', 'now') GROUP BY uuid ORDER BY count DESC LIMIT 3", map);
        }).abortIfNull(() -> fc.complete(null)).nextTask(result -> {
            HashMap<UUID, Integer> top = new HashMap<>();
            for(HashMap<String, Object> map : result) {
                UUID uuid = UUID.fromString((String) map.get("uuid"));
                int points = (int) map.get("count");
                if(points < 0)
                    continue;

                top.put(uuid, points);
            }

            return top;
        }).lastTask(fc::complete).setFullExecute(true).execute();
        return fc;
    }

    public static CompletableFuture<HashMap<UUID, Integer>> getTopTotal(Attraction attraction) {
        CompletableFuture<HashMap<UUID, Integer>> fc = new CompletableFuture<>();
        if(attraction == null) {
            fc.complete(null);
            return fc;
        }

        if(!API.isAttraction(attraction.getId())) {
            System.out.println("[ThemePark] Unknown attraction with ID: "+attraction.getId());
            fc.complete(null);
            return fc;
        }

        Work.firstTask(() -> {
            DB db = DBManager.getDatabase("data");
            if(db == null)
                return null;

            HashMap<Integer, Object> map = new HashMap<>();
            map.put(1, attraction.getId());

            return db.executeQuery("SELECT uuid, SUM(points) AS count FROM counts WHERE attraction_id=? GROUP BY uuid ORDER BY count DESC LIMIT 3", map);
        }).abortIfNull(() -> fc.complete(null)).nextTask(result -> {
            HashMap<UUID, Integer> top = new HashMap<>();
            for(HashMap<String, Object> map : result) {
                UUID uuid = UUID.fromString((String) map.get("uuid"));
                int points = (int) map.get("count");
                if(points < 0)
                    continue;

                top.put(uuid, points);
            }

            return top;
        }).lastTask(fc::complete).setFullExecute(true).execute();
        return fc;
    }

    private static String fixUUID(String uuid) {
        if(uuid == null || uuid.isEmpty())
            return null;

        StringBuilder sb = new StringBuilder(uuid);
        sb.insert(8, "-");
        sb = new StringBuilder(sb.toString());
        sb.insert(13, "-");
        sb = new StringBuilder(sb.toString());
        sb.insert(18, "-");
        sb = new StringBuilder(sb.toString());
        sb.insert(23, "-");

        return sb.toString();
    }

    private static JsonObject sendGETRequestJSON(String uri) throws IOException {
        URL url = new URL(uri);
        HttpURLConnection con = (HttpURLConnection) url.openConnection();
        con.setRequestMethod("GET");
        con.setRequestProperty("User-Agent", "Mozilla/5.0");
        int code = con.getResponseCode();
        if (code == 404)
            return null;

        BufferedReader in = new BufferedReader(new InputStreamReader(con.getInputStream()));
        String inputLine;
        StringBuilder response = new StringBuilder();
        while ((inputLine = in.readLine()) != null)
            response.append(inputLine);

        in.close();
        JsonParser parser = new JsonParser();
        return parser.parse(response.toString()).getAsJsonObject();
    }

    private static boolean versionCompare(String s1, String s2) {
        String[] thisParts = s1.split("\\.");
        String[] thatParts = s2.split("\\.");

        int length = Math.max(thisParts.length, thatParts.length);
        for (int i = 0; i < length; i++) {
            int thisPart = i < thisParts.length ? Integer.parseInt(thisParts[i]) : 0;
            int thatPart = i < thatParts.length ? Integer.parseInt(thatParts[i]) : 0;
            if(thisPart < thatPart)
                return false;
            if(thisPart > thatPart)
                return true;
        }
        return true;
    }

}
